;**********************************************************************
;    Filename:	    usbtime.asm                                       *
;    Date:          2003/12/7                                        *
;    File Version:                                                    *
;**********************************************************************

	list      p=16F84             ; list directive to define processor
	#include <p16F84.inc>         ; processor specific variable definitions

	__CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_ON & _XT_OSC

;***** VARIABLE DEFINITIONS
w_temp        EQU     0x0C        ; variable used for context saving 
status_temp   EQU     0x0D        ; variable used for context saving

count_data	EQU	0x10		;1sec keisoku count ram
sec1_data	EQU	0x11		;1sec LED data ram
sec10_data	EQU	0x12		;10sec LED data ram
min1_data	EQU	0x13		;1min LED data ram

wait_data	EQU	0x15		;wait data ram
start_stop	EQU	0x16		;start/stop ram
					;start_stop,0  timer start/stop flg
					;start_stop,1  buzzer flg

key_data	EQU	0x17		;key data ram
					;key_data,2  start/stop key
					;key_data,3  up1 key

key_input	EQU	0x18		;key input ram
					;key_input,2  start/stop key
					;key_input,3  up1 key
taihi		EQU	0x19		;taihi ram

tm0_setdata	EQU	63h		;255-156=99=63H  156*128=19.968msec
cnt_setdata	EQU	32h		;19.968*50=0.9984sec
;**********************************************************************
		ORG     0x000             ; processor reset vector
  		goto    main              ; go to beginning of program

;----------------------------------------------------------------------------------
		ORG     0x004             ; interrupt vector location
		movwf   w_temp            ; save off current W register contents
		movf	STATUS,w          ; move status register into W register
		movwf	status_temp       ; save off contents of STATUS register

		;---------------------------------	
		bcf	INTCON,T0IF
		movlw	tm0_setdata
		movwf	TMR0

		decfsz	count_data,1
		goto	next_int
time_up		movlw	cnt_setdata		
		movwf	count_data
		;--------------------------------		
		btfss	start_stop,0	;start/stop check
		goto	next_int

		call	zero_check
		btfsc	start_stop,1	;buzzer check
		goto	next_int

		movlw	01h
		subwf	sec1_data,1
		btfsc	STATUS,0
		goto	next_int
		movlw	09h
		movwf	sec1_data
		
		movlw	01h
		subwf	sec10_data,1
		btfsc	STATUS,0
		goto	next_int
		movlw	05h
		movwf	sec10_data

		movlw	01h
		subwf	min1_data,1
		btfsc	STATUS,0
		goto	next_int
		movlw	09h
		movwf	min1_data

next_int
		nop
		;----------------------------------
		movf    status_temp,w     ; retrieve copy of STATUS register
		movwf	STATUS            ; restore pre-isr STATUS register contents
		swapf   w_temp,f
		swapf   w_temp,w          ; restore pre-isr W register contents
		retfie                    ; return from interrupt
;---------------------------------------------------------------------------------------
main
		call	port_set
		movlw	0ch		;port initial set
		movwf	PORTA
		movlw	00h
		movwf	PORTB

		call	timer0_set

		movlw	00h		;start/stop <= stop
					;buzzer flg off
		movwf	start_stop
		movlw	0ffh		;key data,key input clear
		movwf	key_data
		movwf	key_input

		movlw	cnt_setdata
		movwf	count_data
		call	sec_set
		call	min_clear

main_loop
;-------------------------------------- key scan 1
		movlw	0ffh		;key input clear
		movwf	key_input
		btfss	PORTA,2		;start/stop key check
		bcf	key_input,2	;start/stop key input ari		
		btfss	PORTA,3		;up1 key check
		bcf	key_input,3	;up1 key input ari
;--------------------------------------- led7_disp
		movf	min1_data,0
		call	led_henkan
		btfsc	sec1_data,0
		iorlw	080h
		movwf	PORTB
;------------------------------------- key scan 2
stsp_check
		btfss	PORTA,2		;start/stop key check
		goto	stsp_next
		bsf	key_data,2	;start/stop key off
		goto	up1_check
stsp_next
		btfsc	key_input,2	;start/stop key input check
		goto	up1_check
		btfss	key_data,2	;start/stop key on check
		goto	up1_check
		bcf	key_data,2	;start/stop key on
		btfss	start_stop,0	;start/stop check
		goto	stsp_next_2
		bcf	start_stop,0	;start/stop <=stop
		nop
		bcf	start_stop,1	;buzzer flg off
		goto	up1_check
stsp_next_2
		bsf	start_stop,0	;start/stop <=start		
;-------------------------------------
up1_check
		btfss	PORTA,3		;up1 key check
		goto	up1_next
		bsf	key_data,3	;up1 key off
		goto	up10_check
up1_next
		btfsc	key_input,3	;up1 key input check
		goto	up10_check
		btfss	key_data,3	;up1 key on check
		goto	up10_check
		bcf	key_data,3	;up1 key on
		btfsc	start_stop,0	;start/stop check
		goto	up10_check
		call	sec_set		;min1_data up
		incf	min1_data,1
		movlw	0ah
		subwf	min1_data,0
		btfss	STATUS,0
		goto	up10_check
		movlw	00h
		movwf	min1_data
;--------------------------------------
up10_check

;---------------------------------------
next_key
;---------------------------------------
buzzer
		btfss	start_stop,1	;buzzer flg check
		goto	buzzer_2
		bsf	PORTA,0		;buzzer on
		goto	buzzer_3
buzzer_2	
		bcf	PORTA,0
buzzer_3
;---------------------------------------
		goto	main_loop
;-------------------------------------------------------------------------------
wait_1		
		movlw	0ffh
		movwf	wait_data
wait_2
		decfsz	wait_data,1
		goto	wait_2
		return
;-------------------------------------------------------------------------------
led_henkan
                movwf   taihi
                movlw   low(led_henkan_table)
                addwf   taihi,1
                movlw   high(led_henkan_table)
                movwf   PCLATH
                addcf   PCLATH,1
                movf    taihi,0
                movwf   PCL
led_henkan_table   
		retlw	3fh
		retlw	06h
		retlw	5bh
		retlw	4fh
		retlw	66h
		retlw	6dh
		retlw	7dh
		retlw	27h
		retlw	7fh
		retlw	6fh
		retlw	40h
		retlw	40h
		retlw	40h
		retlw	40h
		retlw	40h
		retlw	40h

;-------------------------------------------------------------------------------
port_set				;RA0,RA1,RA4(OUT)
					;RA2,RA3(IN)
					;RB0,RB1,RB2,RB3,RB4,RB5,RB6,RB7(OUT)
		bsf	STATUS,RP0
		clrf	TRISA
		bsf	TRISA,3
		nop
		bsf	TRISA,2	
		clrf	TRISB
		bcf	STATUS,RP0
		return
;-------------------------------------------------------------------------------
timer0_set
		bsf	STATUS,RP0
		movlw	086h		;(1)PORTB NOT PULLUP
					;(0)INTEDG
					;(0)TOCS
					;(0)TOSE
					;(0)PSA
					;(110)1/128
		movwf	OPTION_REG
		bcf	STATUS,RP0
		
		movlw	tm0_setdata
		movwf	TMR0
		bsf	INTCON,T0IE
		bsf	INTCON,GIE
		return
;------------------------------------------------------------------------------
sec_set
		movlw	09h
		movwf	sec1_data
		movlw	05h
		movwf	sec10_data
		return
;------------------------------------------------------------------------------
min_clear
		movlw	00h
		movwf	min1_data
		return
;------------------------------------------------------------------------------
zero_check
		movlw	09h
		subwf	sec1_data,0
		btfss	STATUS,0
		return
		movlw	05h
		subwf	sec10_data,0
		btfss	STATUS,0
		return
		movf	min1_data,1
		btfss	STATUS,2	;Z check
		return
		btfss	start_stop,0	;start/stop check
		return
		bsf	start_stop,1	;buzzer flg on
		return
;-----------------------------------------------------------------------------
		END                     ; directive 'end of program'

